@@ -1,5 +1,4 @@
 import json
-import logging
 import os
 import queue
 import shutil
@@ -8,17 +7,14 @@ import threading
 import time
 import traceback
 import uuid
-import bottle
-import bottle_websocket
 import jinja2
 import typing
-from gevent import pywsgi
-from geventwebsocket.handler import WebSocketHandler
-from geventwebsocket.logging import create_logger
+from werkzeug.serving import make_server
+
 from .lib import Browser
 from .pyhtmlguiInstance import PyHtmlGuiInstance
 from .view import PyHtmlView
-
+import flask, flask_sock
 
 class PyHtmlGui:
     def __init__(self,
@@ -64,6 +60,7 @@ class PyHtmlGui:
                                 This is the default, so one instance of view_class is shared by all connected frontends
         """
 
+
         self.view_class = view_class
         self.app_instance = app_instance
         self.static_dir = os.path.abspath(static_dir)
@@ -120,11 +117,16 @@ class PyHtmlGui:
         self._token_cookie = "%s" % uuid.uuid4()
         self._token_csrf = "%s" % uuid.uuid4()
 
-        bottle.route("/")(self._main_html)
-        bottle.route("/static/<path:path>")(self._static)
-        bottle.route("/ws", apply=[bottle_websocket.websocket])(self._websocket)
+        self.flaskApp = flask.Flask("pyhtmlgui", static_folder=None, template_folder=None)
+        self.flaskWebsocket =  flask_sock.Sock()
+
+        self.flaskApp.route("/")(self._main_html)
+        self.flaskApp.route("/static/<path:path>")(self._static)
+        self.flaskWebsocket.route("/ws")(self._websocket)
+        self.flaskWebsocket.init_app(self.flaskApp)
+
+        self._server = WebsocketServerThread(self.flaskApp, self.listen_host, self.listen_port)
         self._browser = None
-        self._server = WebsocketServer(host=self.listen_host, port=self.listen_port)
 
         self._file_monitoring = {}
         if self.auto_reload is True:
@@ -134,12 +136,15 @@ class PyHtmlGui:
     def start(self, show_frontend: bool = False, block: bool = True) -> None:
         if show_frontend is True:
             self.show()
-        self._server.start(block)
+        if block is True:
+            self._server.run()
+        else:
+            self._server.start()
 
     def stop(self) -> None:
         # noinspection PyBroadException
         try:
-            self._server.stop()
+            self._server.shutdown()
         except Exception:
             pass
 
@@ -163,53 +168,58 @@ class PyHtmlGui:
             self._browser = Browser(self.browser, self.executable)
         self._browser.open(args, env=env)
 
-    # /main.html
+    # /
     def _main_html(self):
-        if self.shared_secret is not None and bottle.request.query.token != self.shared_secret:
-            return bottle.HTTPResponse(status=403)
+        if self.shared_secret is not None and flask.request.args.get('token') != self.shared_secret:
+            return flask.abort(403)
         template = self._template_env.get_template(self.base_template)
-        response = bottle.HTTPResponse(template.render({
+        response = flask.Response(template.render({
             "csrf_token": self._token_csrf,
             "start_size": json.dumps(self.size),
             "start_position": json.dumps(self.position),
         }))
-        response.set_header('Cache-Control', 'no-store')
-        response.set_header('Set-Cookie', 'token=%s' % self._token_cookie)
+        response.headers['Cache-Control'] = 'no-store'
+        response.headers['Set-Cookie'] = 'token=%s' % self._token_cookie
         return response
 
+
     # /static/<path>
     def _static(self, path: str):
-        if bottle.request.get_cookie("token") != self._token_cookie:
-            return bottle.HTTPResponse(status=403)
-        response = bottle.static_file(path, root=self.static_dir)
-        if path.endswith(".js"):
-            response.set_header("Content-Type", "application/javascript")
-        elif path.endswith(".html"):
-            response.set_header("Content-Type", "text/html")
-        elif path.endswith(".css"):
-            response.set_header("Content-Type", "text/css")
-        elif path.endswith(".jpg") or path.endswith(".jpeg"):
-            response.set_header("Content-Type", "image/jpeg")
-        elif path.endswith(".png"):
-            response.set_header("Content-Type", "image/png")
-        elif path.endswith(".gif"):
-            response.set_header("Content-Type", "image/gif")
-        elif path.endswith(".webp"):
-            response.set_header("Content-Type", "image/webp")
-        elif path.endswith(".svg"):
-            response.set_header("Content-Type", "image/svg")
+        if flask.request.cookies.get('token') != self._token_cookie:
+            return flask.abort(403)
+        response = flask.helpers.send_from_directory(
+            self.static_dir, path, max_age=36000)
 
-        response.set_header("Cache-Control", "public, max-age=36000")
+        if path.endswith(".js"):
+            response.headers["Content-Type"] =  "application/javascript"
+        elif path.endswith(".html"):
+            response.headers["Content-Type"] =  "text/html"
+        elif path.endswith(".css"):
+            response.headers["Content-Type"] =  "text/css"
+        elif path.endswith(".jpg") or path.endswith(".jpeg"):
+            response.headers["Content-Type"] =  "image/jpeg"
+        elif path.endswith(".png"):
+            response.headers["Content-Type"] =  "image/png"
+        elif path.endswith(".gif"):
+            response.headers["Content-Type"] =  "image/gif"
+        elif path.endswith(".webp"):
+            response.headers["Content-Type"] =  "image/webp"
+        elif path.endswith(".svg"):
+            response.headers["Content-Type"] =  "image/svg"
+
+        response.headers["Cache-Control"] = "public, max-age=36000"
         return response
 
     # /ws
     def _websocket(self, ws):
-        if bottle.request.get_cookie("token") != self._token_cookie:
-            return bottle.HTTPResponse(status=403)
-        if bottle.request.query.token != self._token_csrf:
-            return bottle.HTTPResponse(status=403)
-        websocket_connection = WebsocketConnection(ws)
+        if flask.request.cookies.get('token') != self._token_cookie or flask.request.args.get('token') != self._token_csrf:
+            try:
+                ws.close()
+            except:
+                pass
+            return flask.abort(403)
 
+        # get/create Instance
         if self.single_instance is True:
             if len(self._gui_instances) == 0:
                 self._gui_instances.append(PyHtmlGuiInstance(self))
@@ -218,13 +228,16 @@ class PyHtmlGui:
             instance = PyHtmlGuiInstance(self)
             self._gui_instances.append(instance)
 
+        # onConnected Callback
         if self.on_view_connected_callback is not None:
             connections = sum([i.connections_count for i in self._gui_instances]) + 1
             # +1 because connection gets only added in loop later
             self.on_view_connected_callback(len(self._gui_instances), connections)
 
-        instance.websocket_loop(websocket_connection)  # loop while connected
+        #LOOP
+        instance.process(ws)  # loop while connected
 
+        # release/remove Instance
         if instance.connections_count == 0:
             self._gui_instances.remove(instance)
         if hasattr(instance._view, "on_frontend_disconnected"):
@@ -267,73 +280,17 @@ class PyHtmlGui:
                         print(traceback.format_exc())
 
 
-class WebsocketServer(bottle.ServerAdapter):
-    def __init__(self, **options):
-        super().__init__(**options)
-        self.server = None
-        self._server_thread = None
-        self.running = False
+class WebsocketServerThread(threading.Thread):
 
-    def start(self, blocking: bool):
-        if self.running is True:
-            raise Exception("Websocket server already started!")
-        if blocking:
-            self.run(bottle.default_app())
-        else:
-            self._server_thread = threading.Thread(target=self.run, args=[bottle.default_app()], daemon=True)
-            self._server_thread.start()
+    def __init__(self, app, host, port ):
+        threading.Thread.__init__(self)
+        self.server = make_server(host, port, app, threaded=True)
+        self.ctx = app.app_context()
+        self.ctx.push()
 
-    def stop(self):
-        if self.server is not None:
-            self.server.stop()
-
-    def run(self, handler):
-        self.running = True
-        self.server = pywsgi.WSGIServer((self.host, self.port), handler, handler_class=WebSocketHandler)
-        if not self.quiet:
-            self.server.logger = create_logger('geventwebsocket.logging')
-            self.server.logger.setLevel(logging.INFO)
-            self.server.logger.addHandler(logging.StreamHandler())
+    def run(self):
         self.server.serve_forever()
-        self._server_thread = None
-        self.running = False
 
+    def shutdown(self):
+        self.server.shutdown()
 
-class WebsocketConnection:
-    def __init__(self, ws):
-        self.ws = ws
-        self.javascript_call_result_queues = {}
-        self.javascript_call_result_objects = {}
-        self.active = True
-        self.send_queue =queue.Queue()
-        self._send_t = threading.Thread(target=self._send_loop, daemon=True)
-        self._send_t.start()
-
-    def loop(self, callback):
-        while self.active is True:
-            msg = self.ws.receive()
-            if msg is None:
-                break
-            try:
-                callback(json.loads(msg), self.ws)
-            except:
-                continue
-        self.active = False
-        if self.send_queue is not None:
-            self.send_queue.put(None)
-
-    def _send_loop(self):
-        while self.active is True:
-            message = self.send_queue.get()
-            if message is None:
-                break
-            try:
-                self.ws.send(message)
-            except:
-                pass
-        self.active = False
-        self.send_queue = None
-
-    def send(self, message):
-        if self.send_queue is not None:
-            self.send_queue.put(message)
