@@ -1,4 +1,6 @@
 from __future__ import annotations
+
+import threading
 import typing
 import jinja2
 import weakref
@@ -10,7 +12,6 @@ import inspect
 import json
 import queue
 import importlib
-import gevent
 from typing import TYPE_CHECKING
 if TYPE_CHECKING:
     from pyhtmlgui.pyhtmlgui import PyHtmlGui
@@ -79,18 +80,13 @@ class PyHtmlGuiInstance:
             if skip_results is False:
                 websocket_connection.javascript_call_result_objects[self._call_number] = javascript_call_result
                 javascript_call_result.add_call(websocket_connection)
-            websocket_connection.ws.send(data)
+            websocket_connection.send(data)
         return javascript_call_result
 
-    def websocket_loop(self, websocket_connection) -> None:
+    def process(self, ws) -> None:
+        websocket_connection = WebsocketConnection(ws, self._websocket_process_message)
         self._websocket_connections.append(websocket_connection)
-        while True:
-            msg = websocket_connection.ws.receive()
-            if msg is not None:
-                message = json.loads(msg)
-                gevent.spawn(self._websocket_process_message, message, websocket_connection).run()
-            else:
-                break
+        websocket_connection.process()
         self._websocket_connections.remove(websocket_connection)
         if len(self._websocket_connections) == 0:
             self.set_visible(False)
@@ -136,13 +132,13 @@ class PyHtmlGuiInstance:
                 tb = traceback.format_exc()
                 msg = " Exception in: %s(%s)\n" % (function_name, ("%s" % args)[1:-1])
                 msg += " %s" % tb.replace("\n", "\n  ").strip()
-                self.call_javascript("pyhtmlgui.debug_msg", [msg])
+                self.call_javascript("pyhtmlgui.debug_msg", [msg],skip_results=True)
                 print(msg)
                 return_val = None
 
             if not ("skip_results" in message and message["skip_results"] is True):
                 data = json.dumps({'return': message['call'], 'value': return_val}, default=lambda o: None)
-                websocket_connection.ws.send(data)
+                websocket_connection.send(data)
 
         elif 'return' in message:
             call_id = message['return']
@@ -275,6 +271,58 @@ class PyHtmlGuiInstance:
         self._children.remove(child)
 
 
+class WebsocketConnection:
+    def __init__(self, ws, process_msg_callback):
+        self.ws = ws
+        self.process_msg_callback = process_msg_callback
+        self.javascript_call_result_queues = {}
+        self.javascript_call_result_objects = {}
+        self.active = True
+        self.send_queue = queue.Queue()
+        self._send_t = threading.Thread(target=self._send_loop, daemon=True)
+        self._send_t.start()
+
+    def process(self):
+        while self.active is True:
+            try:
+                msg = self.ws.receive()
+            except:
+                msg = None
+            if msg is None:
+                break
+            try:
+                self.process_msg_callback(json.loads(msg), self)
+            except:
+                continue
+        try:
+            self.ws.close()
+        except:
+            pass
+        self.active = False
+        if self.send_queue is not None:
+            self.send_queue.put(None)
+
+    def _send_loop(self):
+        while self.active is True:
+            message = self.send_queue.get()
+            if message is None:
+                break
+            try:
+                self.ws.send(message)
+            except:
+                pass
+        try:
+            self.ws.close()
+        except:
+            pass
+        self.active = False
+        self.send_queue = None
+
+    def send(self, message):
+        if self.send_queue is not None:
+            self.send_queue.put(message)
+
+
 class JavascriptCallResult:
     def __init__(self, call_id):
         self.call_id = call_id
